<?php
/**
 * Suppress the Woo Helper admin notices (e.g. connect to woocommerce.com)
 */
add_filter( 'woocommerce_helper_suppress_admin_notices', '__return_true' );


/**
 * Disallow sellers from using order export plugin.
 */
add_filter( 'woe_current_user_can_export', function() {
	if ( bt_is_seller() || bt_is_subscriber_could_be_seller() ) {
		return false;
	}

	return true;
} );


/**
 * Filter breadcrumb
 */
add_filter( 'woocommerce_get_breadcrumb', function( $crumbs ) {
	// see if the option is set to include Shop page (the page title is provided)
	if ( $shop_page_title = bt_ecomm_option( 'breadcrumb_include_shop_page_title' ) ) {
		// check second item to see if it's the shop page (which is there on the shop page, but not otherwise)
		if ( $crumbs[1][0] != $shop_page_title ) {
			$shop_crumb = [ [ $shop_page_title, wc_get_page_permalink( 'shop' ) ] ];
			array_splice( $crumbs, 1, 0, $shop_crumb );
		}
	}

	//echo '<pre>' . print_r( $crumbs, 1 ) . '</pre>';
	return $crumbs;
} );


/**
 * Use a custom default thumb for image placeholder, from bt-ecomm config.
 */
add_filter( 'woocommerce_placeholder_img_src', function ( $src ) {
	if ( $custom_src = bt_ecomm_option( 'placeholder_img_src' ) ) {
		$src = $custom_src;
	}
	//error_log( ' woocommerce_placeholder_img_src FILTER -- USING $src = ' . $src );
	return $src;
}, 9988, 1);


/* ----- helpers ----- */


/**
 * Are we using tips? Could be set in bt-sellers or bt-ecomm.
 *
 * @return bool
 */
function bt_use_tips() {
	return bt_use_sellers( 'use_tips' ) || bt_ecomm_option( 'use_tips' );
}


/**
 * Do we require customers to be 21 years old? Could be set in bt-sellers or bt-ecomm.
 *
 * @return bool
 */
function bt_require_21() {
	return bt_use_sellers( 'require_21' ) || bt_ecomm_option( 'require_21' );
}


/**
 * Helper to determine if a product is already in the shopping cart.
 *
 * @param int $id
 *
 * @return bool True if item is in cart, false otherwise.
 */
function bt_item_in_cart( $id ) {
	// make sure we have woo
	if ( ! class_exists( 'WC_Cart' ) ) {
		return false;
	}

	// loop items in cart
	foreach ( WC()->cart->get_cart() as $cart_item ) {
		// look for main ID match
		if ( $cart_item['product_id'] == $id || $cart_item['variation_id'] == $id ) {
			return true;
		}
	}

	// didn't find it
	return false;
}


/**
 * Helper to determine quantity for a product in the shopping cart.
 *
 * @param int $id
 *
 * @return bool True if item is in cart, false otherwise.
 */
function bt_item_qty_in_cart( $id ) {
	// make sure we have woo
	if ( ! class_exists( 'WC_Cart' ) ) {
		return 0;
	}

	// loop items in cart
	foreach ( WC()->cart->get_cart() as $cart_item ) {
		// look for main ID match
		if ( $cart_item['product_id'] == $id || $cart_item['variation_id'] == $id ) {
			return $cart_item['quantity'];
		}
	}

	// didn't find it
	return 0;
}


/**
 * Helper to see if given date parts equate to a birthdate 21 years ago or more.
 *
 * @param $month
 * @param $day
 * @param $year
 *
 * @return bool
 */
function bt_customer_is_21( $month, $day, $year ) {
	if ( ! empty( $month ) && ! empty( $day ) && ! empty( $year ) ) {
		$birthdate            = strtotime( $month . ' ' . $day . ', ' . $year );
		$twenty_one_years_ago = strtotime( '21 years ago' );
		//error_log( "AGE CHECK: date = $birthdate / 21ago = $twenty_one_years_ago / diff = " . ($birthdate - $twenty_one_years_ago) );
		return $birthdate <= $twenty_one_years_ago;
	}

	return false;
}


/**
 * Return the current birthdate saved in the cart session, or string like Seller or Admin if current user is one.
 * Used in html data to know whether to open age verification modal.
 *
 * @return string
 */
function bt_cart_birthdate() {
	//error_log( __METHOD__ . ': cart session = ' . print_r( WC()->session, 1 ) );

	if ( ! empty( WC()->session->get( 'birthdate_month' ) )
	     && ! empty( WC()->session->get( 'birthdate_day' ) )
	     && ! empty( WC()->session->get( 'birthdate_year' ) )
	) {
		//error_log( __METHOD__ . ': birthdate_year from CART SESSION = ' . WC()->session->get( 'birthdate_year' ) );
		return WC()->session->get( 'birthdate_month' )
		       . ' ' . WC()->session->get( 'birthdate_day' )
		       . ', ' . WC()->session->get( 'birthdate_year' );

	} else if ( ! empty( $signed_in_user_id = get_current_user_id() ) ) {
		// see if user has birthdate data in account and use that if so
		$birthdate_month = get_user_meta( $signed_in_user_id, 'birthdate_month', true );
		$birthdate_day = get_user_meta( $signed_in_user_id, 'birthdate_day', true );
		$birthdate_year = get_user_meta( $signed_in_user_id, 'birthdate_year', true );
		if ( ! empty( $birthdate_month ) && ! empty( $birthdate_day ) && ! empty( $birthdate_year ) ) {
			WC()->session->set( 'birthdate_month', $birthdate_month );
			WC()->session->set( 'birthdate_day', $birthdate_day );
			WC()->session->set( 'birthdate_year', $birthdate_year );
			//error_log( __METHOD__ . ': birthdate_year from USER META = ' . WC()->session->get( 'birthdate_year' ) );
			return $birthdate_month . ' ' . $birthdate_day . ', ' . $birthdate_year;
		} else if ( bt_is_seller() || bt_is_subscriber_could_be_seller() ) {
			return 'Seller';
		} else if ( bt_can_admin_marketplace() ) {
			return 'Admin';
		}
		//error_log( __METHOD__ . ': SIGNED IN WITHOUT USER META' );

	} else if ( is_account_page() ) {
		// prevent age gate on account page so people can sign in to have age from account set to session
		//error_log( __METHOD__ . ': birthdate_year TBD in MY ACCOUNT' );
		return 'TBD';

	}

	//error_log( __METHOD__ . ': no birthdate in session.' );
	return '';
}


/**
 * AJAX responder to check current cart birthdate.
 *
 * It echoes a string with current birthdate or some other indicator (e.g. admin or seller or TBD) that the
 * age gate shouldn't open.
 */
add_action( 'wp_ajax_bt_current_cart_birthdate', 'bt_current_cart_birthdate' );
add_action( 'wp_ajax_nopriv_bt_current_cart_birthdate', 'bt_current_cart_birthdate' );
function bt_current_cart_birthdate() {
	$birthdate = bt_cart_birthdate();
	//error_log( __METHOD__ . ': birthdate = ' . $birthdate );
	echo $birthdate;
	wp_die();
}


/**
 * AJAX responder to verify age of 21 years or more.
 *
 * It echoes a string of JSON with a success bool and result html.
 */
add_action( 'wp_ajax_bt_age_verify', 'bt_age_verify' );
add_action( 'wp_ajax_nopriv_bt_age_verify', 'bt_age_verify' );
function bt_age_verify() {
	$data = [
		'success' => false,
		'result_html' => '<div class="alert alert-danger">You must be 21+ years of age to visit this site.</div>',
	];

	//error_log( __METHOD__ . ': request = ' . print_r( $_REQUEST, 1 ) );
	//error_log( __METHOD__ . ': cart session = ' . print_r( WC()->session, 1 ) );

	if ( bt_customer_is_21( $_REQUEST['month'], $_REQUEST['day'], $_REQUEST['year'] ) ) {
		// set response data
		$data['success'] = true;
		$data['result_html'] = '<div class="alert alert-success">Thank you!</div>';
		// set birthdate data to cart session
		if ( ! WC()->session->has_session() ) {
			// set customer session cookie since there isn't one yet, which will allow custom session data to persist
			WC()->session->set_customer_session_cookie( true );
		}
		WC()->session->set( 'birthdate_month', $_REQUEST['month'] );
		WC()->session->set( 'birthdate_day', $_REQUEST['day'] );
		WC()->session->set( 'birthdate_year', $_REQUEST['year'] );
		//error_log( __METHOD__ . ': cart session = ' . print_r( WC()->session, 1 ) );
	}

	echo json_encode( $data );
	wp_die();
}


function bt_birthdate_select_options( $which = '' ) {
	$options = [];

	//error_log( __METHOD__ . ': which = ' . $which );

	switch ( $which ) {
		case 'month':
			$options = [
				''          => '-',
				'January'   => 'Jan',
				'February'  => 'Feb',
				'March'     => 'Mar',
				'April'     => 'Apr',
				'May'       => 'May',
				'June'      => 'Jun',
				'July'      => 'Jul',
				'August'    => 'Aug',
				'September' => 'Sep',
				'October'   => 'Oct',
				'November'  => 'Nov',
				'December'  => 'Dec',
			];
			break;

		case 'day':
			$options[''] = '-';
			$number  = 1;
			while ( $number < 32 ) {
				$options[ $number ] = $number;
				$number ++;
			}			break;

		case 'year':
			$current_year = intval( date( 'Y' ) );
			$options[''] = '-';
			$number  = $current_year - 21;
			$limit   = $current_year - 101;
			while ( $number >= $limit ) {
				$options[ $number ] = $number;
				$number --;
			}
			break;
	}

	return $options;
}


/**
 * Build and return html for info about the order being a gift, and its recipient, if it's a gift.
 *
 * @param string|int $order_id
 *
 * @return string
 */
function bt_order_gift_recipient_html( $order_id ) {
	$html = '';

	if ( get_post_meta( $order_id, 'is_a_gift', true ) ) {
		/* SKIPPING THIS SINCE IT'S ELSEWHERE IN THE DETAILS AND WASN'T WORKING INITIALLY.
		$order_comments = get_post_meta( $order_id, 'order_comments', true );
					' . ( $order_comments ? 'Instructions or message: ' . $order_comments : '' ) . '
		*/
		$gift_recipient_name = get_post_meta( $order_id, 'gift_recipient_name', true );
		$gift_recipient_phone = get_post_meta( $order_id, 'gift_recipient_phone', true );
		$html .= '
			<div class="alert alert-info">
				<p>
					<strong>This order is a gift.</strong><br>
					Recipient name: ' . $gift_recipient_name . '<br>
					Recipient phone: <a href="tel:' . $gift_recipient_phone . '">' . $gift_recipient_phone . '</a><br>
				</p>
			</div>
		';
	}

	return $html;
}


/* ----- product single and archive items ----- */

/**
 * For product singles, if we have single-page shop for seller, redirect to the item on the seller page
 */
add_action( 'woocommerce_before_single_product', function() {
	// check if we use single-page shop
	if ( bt_use_sellers( 'seller_single_page_shop' ) ) {
		global $product;
		if ( ! empty( $product ) ) {
			// get the seller
			$seller = bt_get_seller_post( bt_get_author_id_for_post( $product->get_id() ) );
			if ( empty( $seller ) ) {
				// to archive of all sellers
				//error_log( __METHOD__ . ' -- woocommerce_before_single_product' );
				wp_redirect( bt_use_sellers( 'seller_archive_url' ) );
      } else if (
        // we don't allow product singles (which we did for NMA membership before product singles for sellers' items)
        ! is_array( bt_use_sellers( 'seller_single_page_shop_allowed_product_singles' ) )
        // or we do allow some product singles, but redirect if the current product isn't one of them
        || ! in_array( $product->get_id(), bt_use_sellers( 'seller_single_page_shop_allowed_product_singles' ) )
      ) {
				// send to seller page and product anchor
				wp_redirect( get_the_permalink( $seller ) . '#product-' . get_the_id( $product ) );
      }
		}
	}
} );


/**
 * For shop archives, if we have single-page shop for seller, redirect to the seller (if cart) or the seller archive
 */
add_action( 'woocommerce_before_shop_loop', function() {
	// check if we use single-page shop
	if ( bt_use_sellers( 'seller_single_page_shop' ) ) {
		// run if on a shop page
		if ( is_shop() || is_product_category() ) {
			// see if we have a seller (active cart)
			$seller = bt_get_cart_seller();
			if ( empty( $seller ) ) {
				// to archive of all sellers
				//error_log( __METHOD__ . ' -- woocommerce_before_shop_loop / is_shop() = ' . ( is_shop() ? 'TRUE' : 'FALSE' ) );
				wp_redirect( bt_use_sellers( 'seller_archive_url' ) );
			} else {
				// to seller for current cart
				wp_redirect( get_the_permalink( $seller ) );
			}
		}

	}
} );
